@@ -7,13 +7,18 @@ Basic usage:
     print(result.categories)
     print(result.detected_by)
 """
-
+import re
+import abc
+import idna
 import socket
 import asyncio
-import aiodns
-from .providers import Provider, BASE_PROVIDERS, DNSBL_CATEGORIES
+import warnings
 
-class DNSBLResult(object):
+import aiodns
+
+from .providers import Provider, BASE_PROVIDERS, BASE_DOMAIN_PROVIDERS
+
+class DNSBLResult:
     """
     DNSBL Result class to keep all info about ip request results.
 
@@ -49,7 +54,6 @@ class DNSBLResult(object):
             # set blacklisted to True if ip is detected with at least one dnsbl
             self.blacklisted = True
             provider_categories = provider.process_response(result.response)
-            assert provider_categories.issubset(DNSBL_CATEGORIES)
             self.categories = self.categories.union(provider_categories)
             self.detected_by[provider.host] = list(provider_categories)
 
@@ -58,7 +62,7 @@ class DNSBLResult(object):
         return "<DNSBLResult: %s %s (%d/%d)>" % (self.addr, blacklisted, len(self.detected_by),
                                                  len(self.providers))
 
-class DNSBLResponse(object):
+class DNSBLResponse:
     """
     DNSBL Response object
     """
@@ -68,19 +72,20 @@ class DNSBLResponse(object):
         self.response = response
         self.error = error
 
-class DNSBLChecker(object):
-    """ Checker for DNSBL lists
+class BaseDNSBLChecker(abc.ABC):
+    """ BASE Checker for DNSBL lists
         Arguments:
             * providers(list) - list of providers (Provider instance or str)
             * timeout(int) - timeout of dns requests will be passed to resolver
             * tries(int) - retry times
     """
 
-    def __init__(self, providers=BASE_PROVIDERS, timeout=5, 
+    def __init__(self, providers=BASE_PROVIDERS, timeout=5,
                  tries=2, concurrency=200, loop=None):
         self.providers = []
         for provider in providers:
-            assert isinstance(provider, Provider)
+            if not isinstance(provider, Provider):
+                raise ValueError('providers should contain only Provider instances')
             self.providers.append(provider)
         if not loop:
             self._loop = asyncio.new_event_loop()
@@ -90,9 +95,10 @@ class DNSBLChecker(object):
         self._resolver = aiodns.DNSResolver(timeout=timeout, tries=tries, loop=self._loop)
         self._semaphore = asyncio.Semaphore(concurrency)
 
-    async def dnsbl_request(self, addr, provider):
+
+    async def dnsbl_request(self, request, provider):
         """
-        Make lookup to dnsbl provider
+        Make lookup to dnsbl provider for ip
         Parameters:
             * addr (string) - ip address to check
             * provider (string) - dnsbl provider
@@ -105,12 +111,7 @@ class DNSBLChecker(object):
         """
         response = None
         error = None
-        try:
-            socket.inet_aton(addr)
-        except socket.error:
-            raise ValueError('wrong ip format')
-        ip_reversed = '.'.join(reversed(addr.split('.')))
-        dnsbl_query = "%s.%s" % (ip_reversed, provider.host)
+        dnsbl_query = "%s.%s" % (self.prepare_query(request), provider.host)
         try:
             async with self._semaphore:
                 response = await self._resolver.query(dnsbl_query, 'A')
@@ -118,42 +119,77 @@ class DNSBLChecker(object):
             if exc.args[0] != 4: # 4: domain name not found:
                 error = exc
 
-        return DNSBLResponse(addr=addr, provider=provider, response=response, error=error)
+        return DNSBLResponse(addr=request, provider=provider, response=response, error=error)
 
-    async def _check_ip(self, addr):
+    @abc.abstractmethod
+    def prepare_query(self, request):
         """
-        Async check ip with dnsbl providers.
-        Parameters:
-            * addr - ip address to check
-
-        Returns:
-            * DNSBLResult object
+        Prepare query to dnsbl
         """
+        return NotImplemented
 
+    async def check_async(self, request):
         tasks = []
         for provider in self.providers:
-            tasks.append(self.dnsbl_request(addr, provider))
+            tasks.append(self.dnsbl_request(request, provider))
         results = await asyncio.gather(*tasks)
-        return DNSBLResult(addr=addr, results=results)
+        return DNSBLResult(addr=request, results=results)
+
+    def check(self, request):
+        return self._loop.run_until_complete(self.check_async(request))
+
+    def bulk_check(self, requests):
+        tasks = []
+        for request in requests:
+            tasks.append(self.check_async(request))
+        return self._loop.run_until_complete(asyncio.gather(*tasks))
+
+
+class DNSBLIpChecker(BaseDNSBLChecker):
+    """
+    Checker for ips
+    """
+    def prepare_query(self, request):
+        try:
+            socket.inet_aton(request)
+        except socket.error:
+            raise ValueError('wrong ip format')
+        return '.'.join(reversed(request.split('.')))
+
+
+
+class DNSBLDomainChecker(BaseDNSBLChecker):
+    """
+    Checker for domains
+    """
+
+    # regex taken from https://regexr.com/3abjr
+    DOMAIN_REGEX = re.compile(r"^((?!-))(xn--)?[a-z0-9][a-z0-9-_]{0,61}[a-z0-9]{0,1}\.(xn--)?([a-z0-9\-]{1,61}|[a-z0-9-]{1,30}\.[a-z]{2,})$")
+
+    def __init__(self, providers=BASE_DOMAIN_PROVIDERS, timeout=5,
+                 tries=2, concurrency=200, loop=None):
+        super().__init__(providers=providers, timeout=timeout,
+                 tries=tries, concurrency=concurrency, loop=loop)
+
+    def prepare_query(self, request):
+        domain_idna = idna.encode(request).decode()
+        if not self.DOMAIN_REGEX.match(domain_idna):
+            raise ValueError('should be valid domain, got %s' % domain_idna)
+        return domain_idna
+
+# COMPAT
+class DNSBLChecker(DNSBLIpChecker):
+    """
+    Will be deprecated, use DNSBLIpChecker
+    """
+    def __init__(self, *args, **kwargs):
+        warnings.warn('deprecated, use DNSBLIpChecker', DeprecationWarning)
+        super().__init__(*args, **kwargs)
 
     def check_ip(self, addr):
-        """
-        Sync check ip with dnsbl providers.
-        Parameters:
-            * addr - ip address to check
-
-        Returns:
-            * DNSBLResult object
-        """
-
-        return self._loop.run_until_complete(self._check_ip(addr))
+        warnings.warn('deprecated, use check method instead', DeprecationWarning)
+        return self.check(addr)
 
     def check_ips(self, addrs):
-        """
-        sync check multiple ips
-        """
-        tasks = []
-        for addr in addrs:
-            tasks.append(self._check_ip(addr))
-        return self._loop.run_until_complete(asyncio.gather(*tasks)) 
-        
+        warnings.warn('deprecated, use bullk check method instead', DeprecationWarning)
+        return self.bulk_check(addrs)
\ No newline at end of file
